@@ -1,10 +1,10 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
- * Copyright (C) 1998-2016 Stuart Parmenter and others, see AUTHORS file.
+ * Copyright (C) 2013-2020 Peter Bloomfield
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
+ * the Free Software Foundation, either version 2 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,7 +13,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 /*
@@ -145,22 +145,22 @@ get_accel_group(GMenuModel * model,
  * Construct a menu-bar for a GtkApplicationWindow that does not use the
  * GApplication's menubar
  *
- * window       the GtkApplicationWindow
- * entries      array of GActionEntry structures
- * n_entries    length of the array
- * ui_file      filename for GtkBuilder input defining a menu named
- *              "menubar"
- * error        GError for returning error information
- * cb_data      user data for GAction callbacks
+ * window        the GtkApplicationWindow
+ * entries       array of GActionEntry structures
+ * n_entries     length of the array
+ * resource_path resource path for GtkBuilder input defining a menu named
+ *               "menubar"
+ * error         GError for returning error information
+ * cb_data       user data for GAction callbacks
  *
- * returns:     the GtkMenuBar
+ * returns:      the GtkMenuBar
  */
 
 GtkWidget *
 libbalsa_window_get_menu_bar(GtkApplicationWindow * window,
                              const GActionEntry   * entries,
                              gint                   n_entries,
-                             const gchar          * ui_file,
+                             const gchar          * resource_path,
                              GError              ** error,
                              gpointer               cb_data)
 {
@@ -171,7 +171,7 @@ libbalsa_window_get_menu_bar(GtkApplicationWindow * window,
     g_action_map_add_action_entries(map, entries, n_entries, cb_data);
 
     builder = gtk_builder_new();
-    if (gtk_builder_add_from_file(builder, ui_file, error)) {
+    if (gtk_builder_add_from_resource(builder, resource_path, error)) {
         GMenuModel *menu_model;
 
         menu_model =
