@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *  
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef __LIBBALSA_MAILBOX_H__
@@ -258,6 +258,9 @@ struct _LibBalsaMailboxClass {
     void (*test_can_reach) (LibBalsaMailbox          * mailbox,
                             LibBalsaCanReachCallback * cb,
                             gpointer                   cb_data);
+    void (*cache_message) (LibBalsaMailbox *mailbox,
+                           guint            msgno,
+                           LibBalsaMessage *message);
 };
 
 LibBalsaMailbox *libbalsa_mailbox_new_from_config(const gchar *prefix,
@@ -619,6 +622,7 @@ guint libbalsa_mailbox_get_open_ref(LibBalsaMailbox * mailbox);
 gboolean libbalsa_mailbox_get_readonly(LibBalsaMailbox * mailbox);
 const gchar * libbalsa_mailbox_get_config_prefix(LibBalsaMailbox * mailbox);
 gboolean libbalsa_mailbox_get_has_unread_messages(LibBalsaMailbox * mailbox);
+gboolean libbalsa_mailbox_get_messages_threaded(LibBalsaMailbox * mailbox);
 
 /*
  * Setters
@@ -628,7 +632,6 @@ void libbalsa_mailbox_set_filters(LibBalsaMailbox * mailbox, GSList * filters);
 void libbalsa_mailbox_set_url(LibBalsaMailbox * mailbox, const gchar * url);
 void libbalsa_mailbox_set_first_unread(LibBalsaMailbox * mailbox, guint first);
 void libbalsa_mailbox_set_msg_tree_changed(LibBalsaMailbox * mailbox, gboolean changed);
-void libbalsa_mailbox_set_is_directory(LibBalsaMailbox * mailbox, gboolean is_directory);
 void libbalsa_mailbox_set_readonly(LibBalsaMailbox * mailbox, gboolean readonly);
 void libbalsa_mailbox_set_no_reassemble(LibBalsaMailbox * mailbox,
                                         gboolean no_reassemble);
@@ -636,6 +639,8 @@ void libbalsa_mailbox_set_name(LibBalsaMailbox * mailbox, const gchar * name);
 void libbalsa_mailbox_set_view(LibBalsaMailbox * mailbox, LibBalsaMailboxView * view);
 void libbalsa_mailbox_set_has_unread_messages(LibBalsaMailbox * mailbox,
                                               gboolean has_unread_messages);
+void libbalsa_mailbox_set_messages_threaded(LibBalsaMailbox * mailbox,
+                                          gboolean messages_threaded);
 void libbalsa_mailbox_set_config_prefix(LibBalsaMailbox * mailbox,
                                         const gchar * config_prefix);
 
