@@ -14,7 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 
 /* SORTING METHOD discussion:
@@ -154,12 +154,13 @@ struct _BalsaIndex {
     BalsaMailboxNode* mailbox_node;
     guint current_msgno;
     guint next_msgno;
-	gboolean current_message_is_deleted:1;
+
+    gboolean current_message_is_deleted:1;
     gboolean prev_message:1;
     gboolean next_message:1;
-    gboolean has_selection_changed_idle:1;
-    gboolean has_mailbox_changed_idle:1;
     gboolean collapsing:1;
+    gboolean expanded:1;
+
     int    filter_no;
     gchar *filter_string; /* Quick view filter string, if any */
 
@@ -168,8 +169,16 @@ struct _BalsaIndex {
     gulong row_collapsed_id;
     gulong selection_changed_id;
 
-	LibBalsaMailboxSearchIter *search_iter;
+    /* Idle handler ids */
+    guint selection_changed_idle_id;
+    guint mailbox_changed_idle_id;
+
+    LibBalsaMailboxSearchIter *search_iter;
     BalsaIndexWidthPreference width_preference;
+
+    /* Ephemera: used by idle handlers */
+    GtkTreeRowReference *reference;
+    guint row_inserted_msgno;
 };
 
 /* Class type. */
@@ -210,41 +219,61 @@ bndx_mbnode_weak_notify(gpointer data, GObject *where_the_object_was)
 static void
 bndx_destroy(GObject * obj)
 {
-    BalsaIndex *index;
+    BalsaIndex *bindex;
 
     g_return_if_fail(obj != NULL);
-    index = BALSA_INDEX(obj);
+    bindex = BALSA_INDEX(obj);
 
-    if (index->mailbox_node) {
+    if (bindex->mailbox_node) {
 	LibBalsaMailbox* mailbox;
 
-	if ((mailbox = balsa_mailbox_node_get_mailbox(index->mailbox_node)) != NULL) {
+	if ((mailbox = balsa_mailbox_node_get_mailbox(bindex->mailbox_node)) != NULL) {
 	    g_signal_handlers_disconnect_matched(mailbox,
 						 G_SIGNAL_MATCH_DATA,
-						 0, 0, NULL, NULL, index);
-	    gtk_tree_view_set_model(GTK_TREE_VIEW(index), NULL);
+						 0, 0, NULL, NULL, bindex);
+	    gtk_tree_view_set_model(GTK_TREE_VIEW(bindex), NULL);
 	    libbalsa_mailbox_close(mailbox, balsa_app.expunge_on_close);
 	}
 
-	if (index->mailbox_node) {
-            g_object_weak_unref(G_OBJECT(index->mailbox_node),
+	if (bindex->mailbox_node) {
+            g_object_weak_unref(G_OBJECT(bindex->mailbox_node),
                                 (GWeakNotify) bndx_mbnode_weak_notify,
-                                index);
-            index->mailbox_node = NULL;
+                                bindex);
+            bindex->mailbox_node = NULL;
         }
     }
 
-    if (index->search_iter) {
-        libbalsa_mailbox_search_iter_unref(index->search_iter);
-        index->search_iter = NULL;
+    if (bindex->search_iter) {
+        libbalsa_mailbox_search_iter_unref(bindex->search_iter);
+        bindex->search_iter = NULL;
     }
 
-    if (index->popup_menu) {
-        g_object_unref(index->popup_menu);
-        index->popup_menu = NULL;
+    if (bindex->popup_menu) {
+        g_object_unref(bindex->popup_menu);
+        bindex->popup_menu = NULL;
     }
 
-    g_free(index->filter_string); index->filter_string = NULL;
+    if (bindex->selection_changed_idle_id != 0) {
+        g_source_remove(bindex->selection_changed_idle_id);
+        bindex->selection_changed_idle_id = 0;
+    }
+
+    if (bindex->mailbox_changed_idle_id != 0) {
+        g_source_remove(bindex->mailbox_changed_idle_id);
+        bindex->mailbox_changed_idle_id = 0;
+    }
+
+    /* Clean up any other idle handler sources */
+    while (g_source_remove_by_user_data(bindex))
+        /* Nothing */ ;
+
+    g_free(bindex->filter_string);
+    bindex->filter_string = NULL;
+
+    if (bindex->reference != NULL) {
+        gtk_tree_row_reference_free(bindex->reference);
+        bindex->reference = NULL;
+    }
 
     G_OBJECT_CLASS(balsa_index_parent_class)->dispose(obj);
 }
@@ -459,25 +488,6 @@ balsa_index_init(BalsaIndex * index)
                      G_CALLBACK(bndx_drag_cb), NULL);
 
     balsa_index_set_column_widths(index);
-    gtk_widget_show_all (GTK_WIDGET(index));
-}
-
-/*
- * Remove a GObject reference; if it was the last reference (and the
- * GObject has now been finalized), clear the location and return TRUE.
- */
-static gboolean
-bndx_clear_if_last_ref(gpointer data)
-{
-    GObject **object = data;
-
-    g_object_add_weak_pointer(G_OBJECT(*object), (gpointer *) data);
-    g_object_unref(*object);
-    if (*object) {
-        g_object_remove_weak_pointer(*object, (gpointer *) data);
-        return FALSE;
-    }
-    return TRUE;
 }
 
 /* Callbacks used by bndx_instance_init. */
@@ -493,30 +503,29 @@ bndx_clear_if_last_ref(gpointer data)
 
 /* idle callback: */
 static gboolean
-bndx_selection_changed_idle(BalsaIndex * index)
+bndx_selection_changed_idle(BalsaIndex * bindex)
 {
     LibBalsaMailbox *mailbox;
     guint msgno;
     GtkTreeSelection *selection;
 
-    if (bndx_clear_if_last_ref(&index))
-        return FALSE;
-    index->has_selection_changed_idle = FALSE;
+    bindex->selection_changed_idle_id = 0;
 
-    if (!index->mailbox_node)
-        return FALSE;
-    mailbox = balsa_mailbox_node_get_mailbox(index->mailbox_node);
-    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(index));
+    if (bindex->mailbox_node == NULL)
+        return G_SOURCE_REMOVE;
+
+    mailbox = balsa_mailbox_node_get_mailbox(bindex->mailbox_node);
+    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(bindex));
 
     /* Save next_msgno, because changing flags may zero it. */
-    msgno = index->next_msgno;
-    if (index->current_msgno) {
+    msgno = bindex->next_msgno;
+    if (bindex->current_msgno) {
         /* The current message has been deselected. */
-        g_signal_handler_block(selection, index->selection_changed_id);
-        libbalsa_mailbox_msgno_change_flags(mailbox, index->current_msgno,
+        g_signal_handler_block(selection, bindex->selection_changed_id);
+        libbalsa_mailbox_msgno_change_flags(mailbox, bindex->current_msgno,
                                             0,
                                             LIBBALSA_MESSAGE_FLAG_SELECTED);
-        g_signal_handler_unblock(selection, index->selection_changed_id);
+        g_signal_handler_unblock(selection, bindex->selection_changed_id);
     }
 
     if (msgno) {
@@ -526,12 +535,12 @@ bndx_selection_changed_idle(BalsaIndex * index)
             msgno = 0;
         else {
             if (!gtk_tree_selection_path_is_selected(selection, path)) {
-                bndx_expand_to_row(index, path);
-                bndx_select_row(index, path);
+                bndx_expand_to_row(bindex, path);
+                bndx_select_row(bindex, path);
             }
             gtk_tree_path_free(path);
 
-            index->current_message_is_deleted =
+            bindex->current_message_is_deleted =
                 libbalsa_mailbox_msgno_has_flags(mailbox, msgno,
                                                  LIBBALSA_MESSAGE_FLAG_DELETED,
                                                  0);
@@ -541,10 +550,10 @@ bndx_selection_changed_idle(BalsaIndex * index)
         }
     }
 
-    index->current_msgno = msgno;
-    bndx_changed_find_row(index);
+    bindex->current_msgno = msgno;
+    bndx_changed_find_row(bindex);
 
-    return FALSE;
+    return G_SOURCE_REMOVE;
 }
 
 /* GtkTreeSelectionForeachFunc: */
@@ -558,19 +567,19 @@ bndx_selection_changed_func(GtkTreeModel * model, GtkTreePath * path,
 
 /* the signal handler: */
 static void
-bndx_selection_changed(GtkTreeSelection * selection, BalsaIndex * index)
+bndx_selection_changed(GtkTreeSelection * selection, BalsaIndex * bindex)
 {
-    index->next_msgno = 0;
+    bindex->next_msgno = 0;
     gtk_tree_selection_selected_foreach(selection,
                                         (GtkTreeSelectionForeachFunc)
                                         bndx_selection_changed_func,
-                                        &index->next_msgno);
+                                        &bindex->next_msgno);
 
-    if (index->current_msgno) {
+    if (bindex->current_msgno) {
         GtkTreePath *path;
 
-        if (libbalsa_mailbox_msgno_find(balsa_mailbox_node_get_mailbox(index->mailbox_node),
-                                        index->current_msgno,
+        if (libbalsa_mailbox_msgno_find(balsa_mailbox_node_get_mailbox(bindex->mailbox_node),
+                                        bindex->current_msgno,
                                         &path, NULL)) {
             gboolean update_preview = TRUE;
 
@@ -578,31 +587,30 @@ bndx_selection_changed(GtkTreeSelection * selection, BalsaIndex * index)
                 /* The current message is still selected; we leave the
                  * preview unchanged. */
                 update_preview = FALSE;
-            } else if (index->collapsing
-                       && !bndx_row_is_viewable(index, path)) {
+            } else if (bindex->collapsing
+                       && !bndx_row_is_viewable(bindex, path)) {
                 /* The current message was deselected because its thread
                  * was collapsed; we leave the preview unchanged, and to
                  * avoid confusion, we unselect all messages. */
                 g_signal_handler_block(selection,
-                                       index->selection_changed_id);
+                                       bindex->selection_changed_id);
                 gtk_tree_selection_unselect_all(selection);
                 g_signal_handler_unblock(selection,
-                                         index->selection_changed_id);
+                                         bindex->selection_changed_id);
                 update_preview = FALSE;
             }
             gtk_tree_path_free(path);
 
             if (!update_preview) {
-                g_signal_emit(index, balsa_index_signals[INDEX_CHANGED], 0);
+                g_signal_emit(bindex, balsa_index_signals[INDEX_CHANGED], 0);
                 return;
             }
         }
     }
 
-    if (!index->has_selection_changed_idle) {
-        index->has_selection_changed_idle = TRUE;
-        g_idle_add((GSourceFunc) bndx_selection_changed_idle,
-                   g_object_ref(index));
+    if (bindex->selection_changed_idle_id == 0) {
+        bindex->selection_changed_idle_id =
+            g_idle_add((GSourceFunc) bndx_selection_changed_idle, bindex);
     }
 }
 
@@ -804,60 +812,71 @@ balsa_index_new(void)
  */
 
 static gboolean
-bndx_scroll_on_open_idle(BalsaIndex *index)
+bndx_scroll_on_open_idle(BalsaIndex *bindex)
 {
     LibBalsaMailbox *mailbox;
     guint first_unread;
-    GtkTreeView *tree_view = GTK_TREE_VIEW(index);
+    GtkTreeView *tree_view = GTK_TREE_VIEW(bindex);
     GtkTreePath *path;
     gpointer view_on_open;
 
-    if (bndx_clear_if_last_ref(&index))
-        return FALSE;
+    mailbox = balsa_mailbox_node_get_mailbox(bindex->mailbox_node);
+
+    if (!libbalsa_mailbox_get_messages_threaded(mailbox))
+        return TRUE; /* G_SOURCE_CONTINUE */
+
+    if (balsa_app.expand_tree &&
+        libbalsa_mailbox_get_threading_type(mailbox) != LB_MAILBOX_THREADING_FLAT &&
+        !bindex->expanded) {
+        gtk_tree_view_expand_all(tree_view);
+        bindex->expanded = TRUE;
+        return TRUE; /* G_SOURCE_CONTINUE */
+    }
 
-    balsa_index_update_tree(index, balsa_app.expand_tree);
-    mailbox = balsa_mailbox_node_get_mailbox(index->mailbox_node);
     first_unread = libbalsa_mailbox_get_first_unread(mailbox);
     if (first_unread > 0) {
-	unsigned msgno = first_unread;
         libbalsa_mailbox_set_first_unread(mailbox, 0);
-        if (!libbalsa_mailbox_msgno_find(mailbox, msgno, &path, NULL))
+        if (!libbalsa_mailbox_msgno_find(mailbox, first_unread, &path, NULL)) {
+            gtk_widget_show(GTK_WIDGET(bindex));
             return FALSE; /* Oops! */
+        }
     } else {
         /* we want to scroll to the last one in current order. The
            alternative which is to scroll to the most recently
            delivered does not feel natural when other sorting order is
            used */
-        int total = gtk_tree_model_iter_n_children
-            (GTK_TREE_MODEL(mailbox), NULL);
-        if(total == 0)
+        gint n_children =
+            gtk_tree_model_iter_n_children(GTK_TREE_MODEL(mailbox), NULL);
+        if (n_children == 0) {
+            gtk_widget_show(GTK_WIDGET(bindex));
             return FALSE;
-        path = gtk_tree_path_new_from_indices(total - 1, -1);
+        }
+        path = gtk_tree_path_new_from_indices(n_children - 1, -1);
     }
 
-    bndx_expand_to_row(index, path);
+    bndx_expand_to_row(bindex, path);
     gtk_tree_view_scroll_to_cell(tree_view, path, NULL, FALSE, 0, 0);
 
     view_on_open =
         g_object_get_data(G_OBJECT(mailbox), BALSA_INDEX_VIEW_ON_OPEN);
     g_object_set_data(G_OBJECT(mailbox), BALSA_INDEX_VIEW_ON_OPEN, NULL);
 
-    if (gtk_tree_view_get_model(tree_view)) {
-        if ((view_on_open && GPOINTER_TO_INT(view_on_open))
-            || balsa_app.view_message_on_open)
-            bndx_select_row(index, path);
-        else {
-            GtkTreeSelection *selection;
-            gulong changed_id = index->selection_changed_id;
+    if ((view_on_open && GPOINTER_TO_INT(view_on_open))
+        || balsa_app.view_message_on_open) {
+        bndx_select_row(bindex, path);
+    } else {
+        GtkTreeSelection *selection;
+        gulong changed_id = bindex->selection_changed_id;
 
-            selection = gtk_tree_view_get_selection(tree_view);
-            g_signal_handler_block(selection, changed_id);
-            gtk_tree_view_set_cursor(tree_view, path, NULL, FALSE);
-            gtk_tree_selection_unselect_all(selection);
-            g_signal_handler_unblock(selection, changed_id);
-        }
+        selection = gtk_tree_view_get_selection(tree_view);
+        g_signal_handler_block(selection, changed_id);
+        gtk_tree_view_set_cursor(tree_view, path, NULL, FALSE);
+        gtk_tree_selection_unselect_all(selection);
+        g_signal_handler_unblock(selection, changed_id);
     }
+
     gtk_tree_path_free(path);
+    gtk_widget_show(GTK_WIDGET(bindex));
 
     return FALSE;
 }
@@ -867,47 +886,39 @@ balsa_index_scroll_on_open(BalsaIndex * bindex)
 {
     /* Scroll in an idle handler, because the mailbox is perhaps being
      * opened in its own idle handler. */
-    g_idle_add((GSourceFunc) bndx_scroll_on_open_idle,
-               g_object_ref(bindex));
+    /* Use low priority, so that GtkTreeView's layout idle handlers get
+     * to finish the layout first. */
+    g_idle_add_full(G_PRIORITY_LOW, (GSourceFunc) bndx_scroll_on_open_idle, bindex, NULL);
 }
 
 static LibBalsaCondition *cond_undeleted;
 
 /* Callback for the mailbox's "row-inserted" signal; queue an idle
  * handler to expand the thread. */
-struct bndx_mailbox_row_inserted_info {
-    LibBalsaMailbox *mailbox;
-    guint msgno;
-    BalsaIndex *index;
-};
 
 static gboolean
-bndx_mailbox_row_inserted_idle(struct bndx_mailbox_row_inserted_info *info)
+bndx_mailbox_row_inserted_idle(BalsaIndex *bindex)
 {
+    GtkTreeView *tree_view = GTK_TREE_VIEW(bindex);
+    GtkTreeModel *model = gtk_tree_view_get_model(tree_view);
+    LibBalsaMailbox *mailbox = LIBBALSA_MAILBOX(model);
     GtkTreePath *path;
-    if (libbalsa_mailbox_msgno_find(info->mailbox, info->msgno,
-                                    &path, NULL)) {
-        bndx_expand_to_row(info->index, path);
+
+    if (libbalsa_mailbox_msgno_find(mailbox, bindex->row_inserted_msgno, &path, NULL)) {
+        bndx_expand_to_row(bindex, path);
         gtk_tree_path_free(path);
     }
-    g_object_unref(info->mailbox);
-    g_object_unref(info->index);
-    g_free(info);
-    return FALSE;
+
+    return G_SOURCE_REMOVE;
 }
 
 static void
 bndx_mailbox_row_inserted_cb(LibBalsaMailbox * mailbox, GtkTreePath * path,
-                             GtkTreeIter * iter, BalsaIndex * index)
+                             GtkTreeIter * iter, BalsaIndex * bindex)
 {
-    guint msgno;
-
     if (libbalsa_mailbox_get_state(mailbox) != LB_MAILBOX_STATE_OPEN)
         return;
 
-    gtk_tree_model_get(GTK_TREE_MODEL(mailbox), iter,
-                       LB_MBOX_MSGNO_COL, &msgno, -1);
-
     if (balsa_app.expand_tree
 #ifdef BALSA_EXPAND_TO_NEW_UNREAD_MESSAGE
         || (balsa_app.expand_to_new_unread
@@ -915,18 +926,13 @@ bndx_mailbox_row_inserted_cb(LibBalsaMailbox * mailbox, GtkTreePath * path,
                                                 LIBBALSA_MESSAGE_FLAG_UNREAD,
                                                 0))
 #endif /* BALSA_EXPAND_TO_NEW_UNREAD_MESSAGE */
-        )
-    {
-	struct bndx_mailbox_row_inserted_info *info =
-	    g_new(struct bndx_mailbox_row_inserted_info, 1);
-	info->mailbox = mailbox;
-	g_object_ref(mailbox);
-	info->index = index;
-	g_object_ref(index);
-	info->msgno = msgno;
-	g_idle_add_full(G_PRIORITY_LOW, /* to run after threading */
-		        (GSourceFunc) bndx_mailbox_row_inserted_idle,
-			info, NULL);
+        ) {
+        gtk_tree_model_get(GTK_TREE_MODEL(mailbox), iter,
+                           LB_MBOX_MSGNO_COL, &bindex->row_inserted_msgno, -1);
+
+        g_idle_add_full(G_PRIORITY_LOW, /* to run after threading */
+                        (GSourceFunc) bndx_mailbox_row_inserted_idle,
+                        bindex, NULL);
     }
 }
 
@@ -945,31 +951,38 @@ bndx_mailbox_message_expunged_cb(LibBalsaMailbox * mailbox, guint msgno,
         --bindex->next_msgno;
 }
 
-/* balsa_index_load_mailbox_node:
+/* balsa_index_set_mailbox_node:
  *
+ * Called in a thread, so do not call Gtk;
  * balsa_mailbox_node_get_mailbox(mbnode) is already open
  */
-
-gboolean
-balsa_index_load_mailbox_node(BalsaIndex * index,
-                              BalsaMailboxNode* mbnode)
+void
+balsa_index_set_mailbox_node(BalsaIndex * bindex,
+                             BalsaMailboxNode* mbnode)
 {
-    GtkTreeView *tree_view;
-    LibBalsaMailbox *mailbox;
-
-    g_return_val_if_fail(BALSA_IS_INDEX(index), TRUE);
-    g_return_val_if_fail(index->mailbox_node == NULL, TRUE);
-    g_return_val_if_fail(BALSA_IS_MAILBOX_NODE(mbnode), TRUE);
-
-    mailbox = balsa_mailbox_node_get_mailbox(mbnode);
-    g_return_val_if_fail(LIBBALSA_IS_MAILBOX(mailbox), TRUE);
+    g_return_if_fail(BALSA_IS_INDEX(bindex));
+    g_return_if_fail(bindex->mailbox_node == NULL);
+    g_return_if_fail(BALSA_IS_MAILBOX_NODE(mbnode));
 
     /*
      * set the new mailbox
      */
-    index->mailbox_node = mbnode;
+    bindex->mailbox_node = mbnode;
     g_object_weak_ref(G_OBJECT(mbnode),
-                      (GWeakNotify) bndx_mbnode_weak_notify, index);
+                      (GWeakNotify) bndx_mbnode_weak_notify, bindex);
+}
+
+
+void
+balsa_index_load_mailbox_node(BalsaIndex * index)
+{
+    LibBalsaMailbox *mailbox;
+    GtkTreeView *tree_view;
+
+    g_return_if_fail(BALSA_IS_INDEX(index));
+
+    mailbox = balsa_mailbox_node_get_mailbox(index->mailbox_node);
+
     /*
      * rename "from" column to "to" for outgoing mail
      */
@@ -1002,8 +1015,6 @@ balsa_index_load_mailbox_node(BalsaIndex * index,
     index->search_iter = libbalsa_mailbox_search_iter_new(cond_undeleted);
     /* Note when this mailbox was opened, for use in auto-closing. */
     balsa_mailbox_node_set_last_use_time(index->mailbox_node);
-
-    return FALSE;
 }
 
 void
@@ -1290,6 +1301,22 @@ balsa_index_get_next_msgno(BalsaIndex * bindex)
  *
  * Note: iter must be valid; it isn't checked here.
  */
+
+static gboolean
+bndx_expand_to_row_and_select_idle(BalsaIndex *bindex)
+{
+    GtkTreePath *path;
+
+    path = gtk_tree_row_reference_get_path(bindex->reference);
+    gtk_tree_row_reference_free(bindex->reference);
+    bindex->reference = NULL;
+
+    bndx_select_row(bindex, path);
+    gtk_tree_path_free(path);
+
+    return G_SOURCE_REMOVE;
+}
+
 static void
 bndx_expand_to_row_and_select(BalsaIndex * index, GtkTreeIter * iter)
 {
@@ -1298,8 +1325,11 @@ bndx_expand_to_row_and_select(BalsaIndex * index, GtkTreeIter * iter)
 
     path = gtk_tree_model_get_path(model, iter);
     bndx_expand_to_row(index, path);
-    bndx_select_row(index, path);
+
+    index->reference = gtk_tree_row_reference_new(model, path);
     gtk_tree_path_free(path);
+
+    g_idle_add((GSourceFunc) bndx_expand_to_row_and_select_idle, index);
 }
 
 /* End of select message interfaces. */
@@ -1348,35 +1378,21 @@ balsa_index_set_column_widths(BalsaIndex * index)
 */
 
 static gboolean
-bndx_queue_draw_idle_cb(gpointer bindex)
+bndx_queue_draw_idle(gpointer data)
 {
-    if (!bndx_clear_if_last_ref(&bindex))
-        gtk_widget_queue_draw(bindex);
+    BalsaIndex *bindex = data;
 
-    return FALSE;
+    gtk_widget_queue_draw(GTK_WIDGET(bindex));
+
+    return G_SOURCE_REMOVE;
 }
 
 static gboolean
 bndx_mailbox_changed_idle(BalsaIndex * bindex)
 {
-    LibBalsaMailbox *mailbox;
-    guint first_unread;
     GtkTreePath *path;
 
-    if (bndx_clear_if_last_ref(&bindex))
-        return FALSE;
-
-    bindex->has_mailbox_changed_idle = FALSE;
-
-    mailbox = balsa_mailbox_node_get_mailbox(bindex->mailbox_node);
-    first_unread = libbalsa_mailbox_get_first_unread(mailbox);
-    if (first_unread > 0
-        && libbalsa_mailbox_msgno_find(mailbox, first_unread,
-                                       &path, NULL)) {
-        bndx_expand_to_row(bindex, path);
-        gtk_tree_path_free(path);
-        libbalsa_mailbox_set_first_unread(mailbox, 0);
-    }
+    bindex->mailbox_changed_idle_id = 0;
 
     if (bndx_find_current_msgno(bindex, &path, NULL)) {
         /* The thread containing the current message may have been
@@ -1387,10 +1403,9 @@ bndx_mailbox_changed_idle(BalsaIndex * bindex)
 
     bndx_changed_find_row(bindex);
 
-    g_idle_add((GSourceFunc) bndx_queue_draw_idle_cb,
-               g_object_ref(bindex));
+    g_idle_add((GSourceFunc) bndx_queue_draw_idle, bindex);
 
-    return FALSE;
+    return G_SOURCE_REMOVE;
 }
 
 static void
@@ -1410,12 +1425,10 @@ bndx_mailbox_changed_cb(LibBalsaMailbox * mailbox, BalsaIndex * bindex)
             bndx_select_next_threaded(bindex);
     }
 
-    if (bindex->has_mailbox_changed_idle)
-        return;
-
-    bindex->has_mailbox_changed_idle = TRUE;
-    g_idle_add((GSourceFunc) bndx_mailbox_changed_idle,
-               g_object_ref(bindex));
+    if (bindex->mailbox_changed_idle_id == 0) {
+        bindex->mailbox_changed_idle_id =
+            g_idle_add((GSourceFunc) bndx_mailbox_changed_idle, bindex);
+    }
 }
 
 static void
@@ -2025,6 +2038,21 @@ sendmsg_window_destroy_cb(GtkWidget * widget, gpointer data)
     balsa_window_enable_continue(balsa_app.main_window);
 }
 
+static gboolean
+bndx_update_tree_idle(BalsaIndex *bindex)
+{
+    GtkTreeIter iter;
+
+    if (bndx_find_current_msgno(bindex, NULL, &iter))
+        bndx_expand_to_row_and_select(bindex, &iter);
+    else
+        balsa_index_ensure_visible(bindex);
+
+    bndx_changed_find_row(bindex);
+
+    return G_SOURCE_REMOVE;
+}
+
 void
 balsa_index_update_tree(BalsaIndex * index, gboolean expand)
 /* Remarks: In the "collapse" case, we still expand current thread to the
@@ -2032,7 +2060,6 @@ balsa_index_update_tree(BalsaIndex * index, gboolean expand)
 	    approach would be to change preview, e.g. to top of thread. */
 {
     GtkTreeView *tree_view = GTK_TREE_VIEW(index);
-    GtkTreeIter iter;
 
     if (expand) {
         g_signal_handler_block(index, index->row_expanded_id);
@@ -2049,12 +2076,7 @@ balsa_index_update_tree(BalsaIndex * index, gboolean expand)
      * overhead is slight
      * select is needed in both cases, as a previous collapse could have
      * deselected the current message */
-    if (bndx_find_current_msgno(index, NULL, &iter))
-        bndx_expand_to_row_and_select(index, &iter);
-    else
-        balsa_index_ensure_visible(index);
-
-    bndx_changed_find_row(index);
+    g_idle_add((GSourceFunc) bndx_update_tree_idle, index);
 }
 
 /* balsa_index_set_thread_messages: public method. */
@@ -2086,7 +2108,6 @@ balsa_index_set_thread_messages(BalsaIndex * index,
         return;
 
     libbalsa_mailbox_set_threading_type(mailbox, threading_type);
-    libbalsa_mailbox_set_threading(mailbox);
     balsa_index_update_tree(index, balsa_app.expand_tree);
 }
 
@@ -2164,29 +2185,18 @@ balsa_index_transfer(BalsaIndex *index, GArray * msgnos,
 
 /* General helpers. */
 static void
-bndx_expand_to_row(BalsaIndex * index, GtkTreePath * path)
+bndx_expand_to_row(BalsaIndex * bindex, GtkTreePath * path)
 {
-    GtkTreePath *tmp;
-    gint i, j;
+    gint depth = gtk_tree_path_get_depth(path);
 
-    if (!gtk_widget_get_window(GTK_WIDGET(index)))
-        return;
+    if (depth > 1) {
+        gint *indices = gtk_tree_path_get_indices(path);
+        gint last_index = indices[depth - 1];
 
-    tmp = gtk_tree_path_copy(path);
-    while (gtk_tree_path_up(tmp) && gtk_tree_path_get_depth(tmp) > 0
-	   && !gtk_tree_view_row_expanded(GTK_TREE_VIEW(index), tmp));
-    /* Now we go from the deepest unexpanded ancestor up to full path */
-
-    if ((i = gtk_tree_path_get_depth(tmp))
-	< (j = gtk_tree_path_get_depth(path) - 1)) {
-	gint *indices = gtk_tree_path_get_indices(path);
-
-	do {
-	    gtk_tree_path_append_index(tmp, indices[i]);
-	    gtk_tree_view_expand_row(GTK_TREE_VIEW(index), tmp, FALSE);
-	} while (++i < j);
+        gtk_tree_path_up(path);
+        gtk_tree_view_expand_to_path(GTK_TREE_VIEW(bindex), path);
+        gtk_tree_path_append_index(path, last_index);
     }
-    gtk_tree_path_free(tmp);
 }
 
 static void
@@ -2375,7 +2385,7 @@ pipe_in_watch(GIOChannel *channel, GIOCondition condition, gpointer data)
 	    pipe->chars_written += chars_written;
 	    break;
 	case G_IO_STATUS_EOF:
-	    printf("pipe_in::write_chars receieved premature EOF %s\n",
+	    printf("pipe_in::write_chars received premature EOF %s\n",
 		   error ? error->message : "unknown");
 	    pipe_data_destroy(pipe);
 	    return FALSE;
@@ -2732,34 +2742,35 @@ balsa_index_ensure_visible(BalsaIndex * index)
     if (!bndx_find_current_msgno(index, &path, NULL)) {
         /* Current message not displayed, make sure that something
            else is... */
-        gtk_tree_view_get_visible_rect(tree_view, &rect);
-        gtk_tree_view_convert_tree_to_widget_coords(tree_view,
-                                                    rect.x, rect.y,
-                                                    &rect.x, &rect.y);
+        /* Was the cursor set? */
+        gtk_tree_view_get_cursor(tree_view, &path, NULL);
+        if (path == NULL) {
+            /* No */
+            gtk_tree_view_get_visible_rect(tree_view, &rect);
+            gtk_tree_view_convert_tree_to_widget_coords(tree_view,
+                                                        rect.x, rect.y,
+                                                        &rect.x, &rect.y);
 
-        if (gtk_tree_view_get_path_at_pos(tree_view, rect.x, rect.y, &path,
-                                          NULL, NULL, NULL)) {
-            /* We have a message in the view, so we do nothing. */
-            gtk_tree_path_free(path);
-            path = NULL;
-        } else {
-            /* Scroll to the last message. */
-            GtkTreeModel *model;
-            gint n_children;
+            if (gtk_tree_view_get_path_at_pos(tree_view, rect.x, rect.y, &path,
+                                              NULL, NULL, NULL)) {
+                /* We have a message in the view, so we do nothing. */
+                gtk_tree_path_free(path);
+                path = NULL;
+            } else {
+                /* Scroll to the last message. */
+                GtkTreeModel *model;
+                gint n_children;
 
-            model = gtk_tree_view_get_model(tree_view);
-            n_children = gtk_tree_model_iter_n_children(model, NULL);
+                model = gtk_tree_view_get_model(tree_view);
+                n_children = gtk_tree_model_iter_n_children(model, NULL);
 
-            if (n_children > 0) {
-                GtkTreeIter iter;
-                gtk_tree_model_iter_nth_child(model, &iter, NULL,
-                                              --n_children);
-                path = gtk_tree_model_get_path(model, &iter);
+                if (n_children > 0)
+                    path = gtk_tree_path_new_from_indices(n_children - 1, -1);
             }
         }
     }
 
-    if (path) {
+    if (path != NULL) {
         gtk_tree_view_scroll_to_cell(tree_view, path, NULL, FALSE, 0, 0);
         gtk_tree_path_free(path);
     }
