@@ -14,7 +14,7 @@
  * GNU General Public License for more details.
  *  
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 
 #if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
@@ -1200,13 +1200,11 @@ bm_widget_new_html(BalsaMessage * bm, LibBalsaMessageBody * mime_body)
 #endif /* defined HAVE_HTML_WIDGET */
 
 #define GRID_ATTACH(g,str,label)                                  \
-    if (str) { GtkWidget *lbl;                                     \
+    if (str) { GtkWidget *lbl;                                    \
         lbl = gtk_label_new(label);                               \
-        gtk_widget_set_halign(lbl, GTK_ALIGN_START);                \
-        gtk_grid_attach(g, lbl, 0, row, 1, 1);                    \
-        lbl = gtk_label_new(str);                                 \
-        gtk_label_set_line_wrap(GTK_LABEL(lbl), TRUE);            \
         gtk_widget_set_halign(lbl, GTK_ALIGN_START);              \
+        gtk_grid_attach(g, lbl, 0, row, 1, 1);                    \
+    	lbl = libbalsa_create_wrap_label(str, FALSE);			  \
         gtk_grid_attach(g, lbl, 1, row, 1, 1);                    \
         row++;                                                    \
     }
