@@ -376,7 +376,7 @@ store_address_add_address(StoreAddressInfo * info,
     entries = g_new(GtkWidget *, NUM_FIELDS);
     info->entries_list = g_list_append(info->entries_list, entries);
 
-    text = internet_address_to_string(ia, FALSE);
+    text = internet_address_to_string(ia, NULL, FALSE);
     address = libbalsa_address_new();
     libbalsa_address_set_full_name(address,
                                    ia->name != NULL ? ia->name :
